<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateObjectsAPIRequest;
use App\Http\Requests\API\UpdateObjectsAPIRequest;
use App\Models\Objects;
use App\Repositories\ObjectsRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use InfyOm\Generator\Criteria\LimitOffsetCriteria;
use Prettus\Repository\Criteria\RequestCriteria;
use Response;

/**
 * Class ObjectsController
 * @package App\Http\Controllers\API
 */

class ObjectsAPIController extends AppBaseController
{
    /** @var  ObjectsRepository */
    private $objectsRepository;

    public function __construct(ObjectsRepository $objectsRepo)
    {
        $this->objectsRepository = $objectsRepo;
    }

    /**
     * @param Request $request
     * @return Response
     *
     * @SWG\Get(
     *      path="/objects",
     *      summary="Get a listing of the Objects.",
     *      tags={"Objects"},
     *      description="Get all Objects",
     *      produces={"application/json"},
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  type="array",
     *                  @SWG\Items(ref="#/definitions/Objects")
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function index(Request $request)
    {
        $this->objectsRepository->pushCriteria(new RequestCriteria($request));
        $this->objectsRepository->pushCriteria(new LimitOffsetCriteria($request));
        $objects = $this->objectsRepository->all();

        return $this->sendResponse($objects->toArray(), 'Объекты успешно получены');
    }

    /**
     * @param CreateObjectsAPIRequest $request
     * @return Response
     *
     * @SWG\Post(
     *      path="/objects",
     *      summary="Store a newly created Objects in storage",
     *      tags={"Objects"},
     *      description="Store Objects",
     *      produces={"application/json"},
     *      @SWG\Parameter(
     *          name="body",
     *          in="body",
     *          description="Objects that should be stored",
     *          required=false,
     *          @SWG\Schema(ref="#/definitions/Objects")
     *      ),
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  ref="#/definitions/Objects"
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function store(CreateObjectsAPIRequest $request)
    {
        $input = $request->all();

        $objects = $this->objectsRepository->create($input);

        return $this->sendResponse($objects->toArray(), 'Объект успешно добавлен');
    }

    /**
     * @param int $id
     * @return Response
     *
     * @SWG\Get(
     *      path="/objects/{id}",
     *      summary="Display the specified Objects",
     *      tags={"Objects"},
     *      description="Get Objects",
     *      produces={"application/json"},
     *      @SWG\Parameter(
     *          name="id",
     *          description="id of Objects",
     *          type="integer",
     *          required=true,
     *          in="path"
     *      ),
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  ref="#/definitions/Objects"
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function show($id)
    {
        /** @var Objects $objects */
        $objects = $this->objectsRepository->findWithoutFail($id);

        if (empty($objects)) {
            return $this->sendError('Объект не найден');
        }

        return $this->sendResponse($objects->toArray(), 'Объект успешно получен');
    }

    /**
     * @param int $id
     * @param UpdateObjectsAPIRequest $request
     * @return Response
     *
     * @SWG\Put(
     *      path="/objects/{id}",
     *      summary="Update the specified Objects in storage",
     *      tags={"Objects"},
     *      description="Update Objects",
     *      produces={"application/json"},
     *      @SWG\Parameter(
     *          name="id",
     *          description="id of Objects",
     *          type="integer",
     *          required=true,
     *          in="path"
     *      ),
     *      @SWG\Parameter(
     *          name="body",
     *          in="body",
     *          description="Objects that should be updated",
     *          required=false,
     *          @SWG\Schema(ref="#/definitions/Objects")
     *      ),
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  ref="#/definitions/Objects"
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function update($id, UpdateObjectsAPIRequest $request)
    {
        $input = $request->all();

        /** @var Objects $objects */
        $objects = $this->objectsRepository->findWithoutFail($id);

        if (empty($objects)) {
            return $this->sendError('Объект не найден');
        }

        $objects = $this->objectsRepository->update($input, $id);

        return $this->sendResponse($objects->toArray(), 'Объект успешно сохранен');
    }

    /**
     * @param int $id
     * @return Response
     *
     * @SWG\Delete(
     *      path="/objects/{id}",
     *      summary="Remove the specified Objects from storage",
     *      tags={"Objects"},
     *      description="Delete Objects",
     *      produces={"application/json"},
     *      @SWG\Parameter(
     *          name="id",
     *          description="id of Objects",
     *          type="integer",
     *          required=true,
     *          in="path"
     *      ),
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  type="string"
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function destroy($id)
    {
        /** @var Objects $objects */
        $objects = $this->objectsRepository->findWithoutFail($id);

        if (empty($objects)) {
            return $this->sendError('Объект не найден');
        }

        $objects->delete();

        return $this->sendResponse($id, 'Объект успешно удален');
    }
}
