<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateOwnersAPIRequest;
use App\Http\Requests\API\UpdateOwnersAPIRequest;
use App\Models\Owners;
use App\Repositories\OwnersRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use InfyOm\Generator\Criteria\LimitOffsetCriteria;
use Prettus\Repository\Criteria\RequestCriteria;
use Response;

/**
 * Class OwnersController
 * @package App\Http\Controllers\API
 */

class OwnersAPIController extends AppBaseController
{
    /** @var  OwnersRepository */
    private $ownersRepository;

    public function __construct(OwnersRepository $ownersRepo)
    {
        $this->ownersRepository = $ownersRepo;
    }

    /**
     * @param Request $request
     * @return Response
     *
     * @SWG\Get(
     *      path="/owners",
     *      summary="Get a listing of the Owners.",
     *      tags={"Owners"},
     *      description="Get all Owners",
     *      produces={"application/json"},
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  type="array",
     *                  @SWG\Items(ref="#/definitions/Owners")
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function index(Request $request)
    {
        $this->ownersRepository->pushCriteria(new RequestCriteria($request));
        $this->ownersRepository->pushCriteria(new LimitOffsetCriteria($request));
        $owners = $this->ownersRepository->all();

        return $this->sendResponse($owners->toArray(), 'Владельцы успешно найдены');
    }

    /**
     * @param CreateOwnersAPIRequest $request
     * @return Response
     *
     * @SWG\Post(
     *      path="/owners",
     *      summary="Store a newly created Owners in storage",
     *      tags={"Owners"},
     *      description="Store Owners",
     *      produces={"application/json"},
     *      @SWG\Parameter(
     *          name="body",
     *          in="body",
     *          description="Owners that should be stored",
     *          required=false,
     *          @SWG\Schema(ref="#/definitions/Owners")
     *      ),
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  ref="#/definitions/Owners"
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function store(CreateOwnersAPIRequest $request)
    {
        $input = $request->all();

        $owners = $this->ownersRepository->create($input);

        return $this->sendResponse($owners->toArray(), 'Владелец успешно добавлен');
    }

    /**
     * @param int $id
     * @return Response
     *
     * @SWG\Get(
     *      path="/owners/{id}",
     *      summary="Display the specified Owners",
     *      tags={"Owners"},
     *      description="Get Owners",
     *      produces={"application/json"},
     *      @SWG\Parameter(
     *          name="id",
     *          description="id of Owners",
     *          type="integer",
     *          required=true,
     *          in="path"
     *      ),
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  ref="#/definitions/Owners"
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function show($id)
    {
        /** @var Owners $owners */
        $owners = $this->ownersRepository->findWithoutFail($id);

        if (empty($owners)) {
            return $this->sendError('Владелец не найден');
        }

        return $this->sendResponse($owners->toArray(), 'Владелец успешно найден');
    }

    /**
     * @param int $id
     * @param UpdateOwnersAPIRequest $request
     * @return Response
     *
     * @SWG\Put(
     *      path="/owners/{id}",
     *      summary="Update the specified Owners in storage",
     *      tags={"Owners"},
     *      description="Update Owners",
     *      produces={"application/json"},
     *      @SWG\Parameter(
     *          name="id",
     *          description="id of Owners",
     *          type="integer",
     *          required=true,
     *          in="path"
     *      ),
     *      @SWG\Parameter(
     *          name="body",
     *          in="body",
     *          description="Owners that should be updated",
     *          required=false,
     *          @SWG\Schema(ref="#/definitions/Owners")
     *      ),
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  ref="#/definitions/Owners"
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function update($id, UpdateOwnersAPIRequest $request)
    {
        $input = $request->all();

        /** @var Owners $owners */
        $owners = $this->ownersRepository->findWithoutFail($id);

        if (empty($owners)) {
            return $this->sendError('Владелец не найден');
        }

        $owners = $this->ownersRepository->update($input, $id);

        return $this->sendResponse($owners->toArray(), 'Владелец успешно отредактирован');
    }

    /**
     * @param int $id
     * @return Response
     *
     * @SWG\Delete(
     *      path="/owners/{id}",
     *      summary="Remove the specified Owners from storage",
     *      tags={"Owners"},
     *      description="Delete Owners",
     *      produces={"application/json"},
     *      @SWG\Parameter(
     *          name="id",
     *          description="id of Owners",
     *          type="integer",
     *          required=true,
     *          in="path"
     *      ),
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  type="string"
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function destroy($id)
    {
        /** @var Owners $owners */
        $owners = $this->ownersRepository->findWithoutFail($id);

        if (empty($owners)) {
            return $this->sendError('Владелец не найден');
        }

        $owners->delete();

        return $this->sendResponse($id, 'Владелец успешно удален');
    }
}
