<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateWaybillsAPIRequest;
use App\Http\Requests\API\UpdateWaybillsAPIRequest;
use App\Models\Waybills;
use App\Repositories\WaybillsRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use InfyOm\Generator\Criteria\LimitOffsetCriteria;
use Prettus\Repository\Criteria\RequestCriteria;
use Response;

/**
 * Class WaybillsController
 * @package App\Http\Controllers\API
 */

class WaybillsAPIController extends AppBaseController
{
    /** @var  WaybillsRepository */
    private $waybillsRepository;

    public function __construct(WaybillsRepository $waybillsRepo)
    {
        $this->waybillsRepository = $waybillsRepo;
    }

    /**
     * @param Request $request
     * @return Response
     *
     * @SWG\Get(
     *      path="/waybills",
     *      summary="Get a listing of the Waybills.",
     *      tags={"Waybills"},
     *      description="Get all Waybills",
     *      produces={"application/json"},
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  type="array",
     *                  @SWG\Items(ref="#/definitions/Waybills")
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function index(Request $request)
    {
        $this->waybillsRepository->pushCriteria(new RequestCriteria($request));
        $this->waybillsRepository->pushCriteria(new LimitOffsetCriteria($request));
        $waybills = $this->waybillsRepository->all();

        return $this->sendResponse($waybills->toArray(), 'Waybills retrieved successfully');
    }

    /**
     * @param CreateWaybillsAPIRequest $request
     * @return Response
     *
     * @SWG\Post(
     *      path="/waybills",
     *      summary="Store a newly created Waybills in storage",
     *      tags={"Waybills"},
     *      description="Store Waybills",
     *      produces={"application/json"},
     *      @SWG\Parameter(
     *          name="body",
     *          in="body",
     *          description="Waybills that should be stored",
     *          required=false,
     *          @SWG\Schema(ref="#/definitions/Waybills")
     *      ),
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  ref="#/definitions/Waybills"
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function store(CreateWaybillsAPIRequest $request)
    {
        $input = $request->all();

        $waybills = $this->waybillsRepository->create($input);

        return $this->sendResponse($waybills->toArray(), 'Waybills saved successfully');
    }

    /**
     * @param int $id
     * @return Response
     *
     * @SWG\Get(
     *      path="/waybills/{id}",
     *      summary="Display the specified Waybills",
     *      tags={"Waybills"},
     *      description="Get Waybills",
     *      produces={"application/json"},
     *      @SWG\Parameter(
     *          name="id",
     *          description="id of Waybills",
     *          type="integer",
     *          required=true,
     *          in="path"
     *      ),
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  ref="#/definitions/Waybills"
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function show($id)
    {
        /** @var Waybills $waybills */
        $waybills = $this->waybillsRepository->findWithoutFail($id);

        if (empty($waybills)) {
            return $this->sendError('Waybills not found');
        }

        return $this->sendResponse($waybills->toArray(), 'Waybills retrieved successfully');
    }

    /**
     * @param int $id
     * @param UpdateWaybillsAPIRequest $request
     * @return Response
     *
     * @SWG\Put(
     *      path="/waybills/{id}",
     *      summary="Update the specified Waybills in storage",
     *      tags={"Waybills"},
     *      description="Update Waybills",
     *      produces={"application/json"},
     *      @SWG\Parameter(
     *          name="id",
     *          description="id of Waybills",
     *          type="integer",
     *          required=true,
     *          in="path"
     *      ),
     *      @SWG\Parameter(
     *          name="body",
     *          in="body",
     *          description="Waybills that should be updated",
     *          required=false,
     *          @SWG\Schema(ref="#/definitions/Waybills")
     *      ),
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  ref="#/definitions/Waybills"
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function update($id, UpdateWaybillsAPIRequest $request)
    {
        $input = $request->all();

        /** @var Waybills $waybills */
        $waybills = $this->waybillsRepository->findWithoutFail($id);

        if (empty($waybills)) {
            return $this->sendError('Waybills not found');
        }

        $waybills = $this->waybillsRepository->update($input, $id);

        return $this->sendResponse($waybills->toArray(), 'Waybills updated successfully');
    }

    /**
     * @param int $id
     * @return Response
     *
     * @SWG\Delete(
     *      path="/waybills/{id}",
     *      summary="Remove the specified Waybills from storage",
     *      tags={"Waybills"},
     *      description="Delete Waybills",
     *      produces={"application/json"},
     *      @SWG\Parameter(
     *          name="id",
     *          description="id of Waybills",
     *          type="integer",
     *          required=true,
     *          in="path"
     *      ),
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  type="string"
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function destroy($id)
    {
        /** @var Waybills $waybills */
        $waybills = $this->waybillsRepository->findWithoutFail($id);

        if (empty($waybills)) {
            return $this->sendError('Waybills not found');
        }

        $waybills->delete();

        return $this->sendResponse($id, 'Waybills deleted successfully');
    }
}
