<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Repositories\ServersRepository;
use App\Repositories\SyncRepository;
use App\Repositories\UsersRepository;
use App\Sdk\SdkManager;
use App\Sdk\WialonPro;
use App\Sdk\Gelios;
use App\Sdk\NullSdk;
use Illuminate\Foundation\Auth\AuthenticatesUsers;
use Illuminate\Http\Request;

class LoginController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Login Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles authenticating users for the application and
    | redirecting them to your home screen. The controller uses a trait
    | to conveniently provide its functionality to your applications.
    |
    */

    use AuthenticatesUsers;

    /**
     * Where to redirect users after login.
     *
     * @var string
     */
    protected $redirectTo = '/';

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest')->except('logout');
    }

    public function showLoginForm(ServersRepository $serversRepository)
    {
        return view('auth.login', ['servers' => $serversRepository->all()]);
    }

    public function username()
    {
        return 'username';
    }

    public function login(Request $request, ServersRepository $serverRepo, UsersRepository $userRepo, SyncRepository $syncRepository)
    {
        $this->validateLogin($request);

        if ($this->hasTooManyLoginAttempts($request)) {
            $this->fireLockoutEvent($request);

            return $this->sendLockoutResponse($request);
        }

        if ((int)($servers_id = $request->get('servers_id', 0)) !== 0) {
            $server = $serverRepo->findWithoutFail($servers_id);

            if (!$server) {
                return $this->sendFailedServersIdResponse($request, 'Сервер не найден');
            }

            $sdk = (new SdkManager())->instance($server);

            if ($sdk === null) {
                return $this->sendFailedServersIdResponse($request, 'Не найден SDK сервера');
            }

            $sdk->setBaseApiUrl();

            if (!$response = $sdk->login($this->credentials($request))) {
                return $this->sendFailedServersIdResponse($request, 'Пользователь не найден на сервере');
            }

            $user = $userRepo->findWhere($request->only($this->username(), 'servers_id'))->first();

            if (!$user) {
                $array = $request->only($this->username(), 'password', 'servers_id');
                $array['password'] = bcrypt($array['password']);
                $array['danger_password'] = encrypt($array['password']);

                $user = $userRepo->create($array);

                if (!$user) {
                    return $this->sendFailedServersIdResponse($request, 'Ошибка создания пользователя');
                }

                $syncType = 'new';
            } else {
                $syncType = 'update';
            }

            $syncRepository->create([
                'type'     => $syncType,
                'users_id' => $user->id,
            ]);

            $userRepo->update([
                'mon_id'          => $response->user->id,
                'password'        => bcrypt($request->get('password')),
                'danger_password' => encrypt($request->get('password')),
            ], $user->id);
        }

        if ($this->attemptLogin($request)) {
            return $this->sendLoginResponse($request);
        }

        $this->incrementLoginAttempts($request);

        return $this->sendFailedLoginResponse($request);
    }

    protected function credentials(Request $request)
    {
        if ((int)$request->get('servers_id', 0) !== 0) {
            return $request->only($this->username(), 'password', 'servers_id');
        }

        return $request->only($this->username(), 'password');
    }

    protected function sendFailedServersIdResponse(Request $request, $message = '')
    {
        $errors = ['servers_id' => $message];

        if ($request->expectsJson()) {
            return response()->json($errors, 422);
        }

        return redirect()->back()
            ->withInput($request->only($this->username(), 'remember'))
            ->withErrors($errors);
    }
}
