<?php

namespace App\Sdk;

use App\Transformers\WialonProDriversTransformer;
use App\Transformers\WialonProUnitsTransformer;
use App\Transformers\WialonProUnitTransformer;
use Carbon\Carbon;
use League\Fractal\Manager;
use League\Fractal\Resource\Collection;
use League\Fractal\Resource\Item;
use Unirest\Request;

final class WialonPro extends TypeSdkAbstract
{
    private $sid;
    private $cacheResponse;
    private $reportResult;
    private $base_api_url = '';
    private $default_params = [];

    public function __construct()
    {
        $this->setName('wialonpro');
        $this->setSid();
    }

    public function login(array $credentials)
    {
        $params = [
            'user'     => $credentials['username'],
            'password' => $credentials['password'],
        ];

        $response = $this->call('core/login', $params);

        if ($response->code !== 200) {
            return false;
        }

        if (property_exists($response->body, 'ssid')) {
            $this->setSid($response->body->ssid);

            return $response->body;
        }

        return false;
    }

    public function logout()
    {
        $this->call('core/logout');
        $this->setSid();
    }

    public function getUnit($id = 0)
    {
        $params = [
            'itemId' => $id,
            'flags'  => '0x00000011',
        ];

        $response = $this->call('core/search_item', $params);

        return (new Manager())->createData(new Item($response->body, new WialonProUnitTransformer()))->toArray();
    }

    public function getUnits()
    {
        $params = [
            'spec'  => [
                'itemsType'     => 'avl_unit',
                'propName'      => 'sys_name',
                'propValueMask' => '*',
                'sortType'      => 'sys_unique_id',
            ],
            'force' => 1,
            'flags' => '0x00000011',
            'from'  => 0,
            'to'    => '0xffffffff',
        ];

        $response = $this->call('core/search_items', $params);

        if ($response->code !== 200) {
            return [];
        }

        $array = json_decode($response->raw_body, true);

        if (isset($array['error']) && $array['error'] !== '') {
            return [];
        }

        if (! isset($array['items'])) {
            return [];
        }

        return (new Manager())->createData(new Collection(collect($array['items']), new WialonProUnitsTransformer()))->toArray();
    }

    /**
     * @param int $value
     * @return mixed
     */
    public function getDriversAndReports($value = 0)
    {
        if (!$this->cacheResponse) {
            $params = [
                'spec'  => [
                    'itemsType'     => 'avl_resource',
                    'propName'      => 'sys_user_creator',
                    'propValueMask' => $value,
                    'sortType'      => 'sys_unique_id',
                ],
                'force' => 1,
                'flags' => '0x00008101',
                'from'  => 0,
                'to'    => '0xffffffff',
            ];

            $response = $this->call('core/search_items', $params);

            if ($response->code !== 200) {
                return false;
            }

            if ((int)$response->body->totalItemsCount === 0) {
                return false;
            }

            $this->cacheResponse = $response->body->items;
        }

        return $this->cacheResponse;
    }

    public function getDrivers($value = 0, $username = '')
    {
        $drivers = [];
        $response = $this->getDriversAndReports($value);

        if (is_array($response)) {
            foreach ($response as $resource) {
                if ($resource->nm === $username) {
                    $drivers = $resource->drvrs;
                }
            }
        }

        return (new Manager())->createData(new Collection(collect($drivers), new WialonProDriversTransformer()))->toArray();
    }

    public function getReport($value = 0, $report = 'waybills')
    {
        $reportTrip = [];
        $response = $this->getDriversAndReports($value);

        if (!$response) {
            return $reportTrip;
        }

        foreach ($response as $resource) {
            $reports = $resource->rep;

            if ($reports === '') {
                continue;
            }

            foreach ($reports as $rep) {
                if ($rep->nm === $report) {
                    $reportTrip = ['res_id' => $resource->id, 'templ_id' => $rep->id];
                }
            }
        }

        return $reportTrip;
    }

    /**
     * @param array $reportTrip
     * @param int $object
     * @param Carbon $startDate
     * @param Carbon $endDate
     * @return mixed
     */
    public function execReport($reportTrip = [], $object = 0, Carbon $startDate, Carbon $endDate)
    {
        $params = [
            'reportResourceId'  => $reportTrip['res_id'],
            'reportTemplateId'  => $reportTrip['templ_id'],
            'reportObjectId'    => $object,
            'reportObjectSecId' => 0,
            'interval'          => [
                'from'  => $startDate->getTimestamp(),
                'to'    => $endDate->getTimestamp(),
                'flags' => 0,
            ],
            'tzOffset'          => 134228528,
            'lang'              => 'ru',
        ];

        $response = $this->call('report/exec_report', $params);

        if ($response->code !== 200) {
            return false;
        }

        if (isset($response->body->error) && !empty($response->body->error)) {
            return false;
        }

        $this->reportResult = $response->body->reportResult;

        return $this->reportResult;
    }

    public function getTablesReport()
    {
        return data_get($this->reportResult, 'tables', []);
    }

    public function getStatsReport()
    {
        return data_get($this->reportResult, 'stats', []);
    }

    public function call($method, array $params = [])
    {
        $json = json_encode($params);

        return Request::get($this->getBaseApiUrl() . '?svc=' . $method . '&params=' . $json . '&ssid=' . $this->sid);
    }

    /**
     * @return string
     */
    protected function getSid()
    {
        return $this->sid;
    }

    /**
     * @param string $sid
     */
    protected function setSid($sid = '')
    {
        $this->sid = $sid;
    }

    /**
     * @return array
     */
    public function getDefaultParams()
    {
        return $this->default_params;
    }

    /**
     * @param array $default_params
     */
    public function setDefaultParams(array $default_params = [])
    {
        $this->default_params = $default_params;
    }

    /**
     * @return string
     */
    public function getBaseApiUrl()
    {
        return $this->base_api_url;
    }

    /**
     * @param string $base_api_url
     */
    public function setBaseApiUrl($base_api_url = '')
    {
        if (empty($base_api_url)) {
            $base_api_url = sprintf('%s://%s%s/ajax.html', $this->getServer()->scheme, $this->getServer()->url, mb_strlen($this->getServer()->port) > 0 ? ':' . $this->getServer()->port : '');
        }

        $this->base_api_url = $base_api_url;
    }
}